/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: L. Zangrando, <zangrando@pd.infn.it>
 *
 */

package org.glite.ce.cream.client.es;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.apache.axis2.AxisFault;
import org.apache.axis2.databinding.types.NonNegativeInteger;
import org.apache.axis2.databinding.types.URI;
import org.apache.axis2.databinding.types.UnsignedLong;
import org.glite.ce.creamapi.ws.es.activityinfo.AccessControlFault;
import org.glite.ce.creamapi.ws.es.activityinfo.InternalBaseFault;
import org.glite.ce.creamapi.ws.es.activityinfo.InvalidParameterFault;
import org.glite.ce.creamapi.ws.es.activityinfo.UnknownActivityIDFault;
import org.glite.ce.creamapi.ws.es.activityinfo.UnknownAttributeFault;
import org.glite.ce.creamapi.ws.es.activityinfo.VectorLimitExceededFault;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityInfoDocument_t;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityInfoItemChoice_type1;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityInfoItem_type0;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityStatusAttribute;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityStatusItemChoice_type1;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityStatusItem_type0;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityStatusState;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityStatus_type0;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ActivityStatus_type1;
import org.glite.ce.creamapi.ws.es.activityinfo.types.GetActivityInfo;
import org.glite.ce.creamapi.ws.es.activityinfo.types.GetActivityInfoResponse;
import org.glite.ce.creamapi.ws.es.activityinfo.types.GetActivityStatus;
import org.glite.ce.creamapi.ws.es.activityinfo.types.GetActivityStatusResponse;
import org.glite.ce.creamapi.ws.es.activityinfo.types.InternalBaseFault_Type;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ListActivities;
import org.glite.ce.creamapi.ws.es.activityinfo.types.ListActivitiesResponse;
import org.glite.ce.creamapi.ws.es.glue.ComputingActivityState_t;
import org.glite.ce.creamapi.ws.es.glue.ComputingActivityType_t;
import org.glite.ce.creamapi.ws.es.glue.DateTime_t;
import org.glite.ce.creamapi.ws.es.glue.Extension_t;
import org.glite.ce.creamapi.ws.es.glue.Extensions_t;
import org.glite.ce.creamapi.ws.es.glue.JobDescription_t;

public class ActivityInfoClient extends ActivityCommand {

    public static void main(String[] args) {
        List<String> options = new ArrayList<String>(8);
        options.add(EPR);
        options.add(PROXY);
        options.add(FROM_DATE);
        options.add(TO_DATE);
        options.add(LIMIT);
        options.add(GET_ACTIVITY_INFO);
        options.add(GET_ACTIVITY_STATUS);
        options.add(LIST_ACTIVITIES);

        new ActivityInfoClient(args, options);
    }

    public ActivityInfoClient(String[] args, List<String> options) throws RuntimeException {
        super(args, options);
    }

    private String checkValue(ComputingActivityState_t[] state) {
        StringBuffer buffer = new StringBuffer("[ ");

        if (state == null) {
            buffer.append("]");
        } else {
            for (int i = 0; i < state.length; i++) {
                buffer.append(state[i].getComputingActivityState_t()).append(", ");
            }

            buffer.replace(buffer.length() - 2, buffer.length(), " ]");
        }
        return buffer.toString();
    }

    private String checkValue(ComputingActivityType_t type) {
        if (type == null) {
            return "N/A";
        }

        return type.getValue();
    }

    private String checkValue(DateTime_t time) {
        if (time == null) {
            return "N/A";
        }

        return time.getDateTime_t().getTime().toString();
    }

    private String checkValue(Integer value) {
        if (value == null) {
            return "N/A";
        }

        return value.toString();
    }

    private String checkValue(JobDescription_t jobDescription) {
        if (jobDescription == null) {
            return "N/A";
        }

        return jobDescription.getJobDescription_t();
    }

    private String checkValue(String value) {
        if (value == null) {
            value = "N/A";
        }

        return value;
    }

    private String checkValue(String[] values) {
        StringBuffer buffer = new StringBuffer("[ ");

        if (values == null) {
            buffer.append("]");
        } else {
            for (int i = 0; i < values.length; i++) {
                buffer.append(values[i]).append(", ");
            }

            buffer.replace(buffer.length() - 2, buffer.length(), " ]");
        }
        return buffer.toString();
    }

    private String checkValue(URI[] values) {
        StringBuffer buffer = new StringBuffer("[ ");

        if (values == null) {
            buffer.append("]");
        } else {
            for (int i = 0; i < values.length; i++) {
                buffer.append(values[i]).append(", ");
            }

            buffer.replace(buffer.length() - 2, buffer.length(), " ]");
        }
        return buffer.toString();
    }

    private String checkValue(UnsignedLong value) {
        if (value == null) {
            return "N/A";
        }

        return value.toString();
    }

    private String checkValue(URI uri) {
        if (uri == null) {
            return "N/A";
        }

        return uri.toString();
    }

    public void execute() {
        if (isListActivities()) {
            ListActivities req = new ListActivities();
            req.setFromDate(getFromDate());
            req.setToDate(getToDate());

            if (getLimit() > 0) {
                req.setLimit(new NonNegativeInteger(""+getLimit()));
            }
            
            ActivityStatus_type1 activityStatus = null;

            for (String statusId : getIdList()) {
                activityStatus = new ActivityStatus_type1();
                activityStatus.setStatus(ActivityStatusState.Factory.fromValue(statusId));

                req.addActivityStatus(activityStatus);
            }

            ListActivitiesResponse response = null;
            try {
                response = getActivityInfoServiceStub().listActivities(req);
            } catch (AxisFault e) {
                System.out.println(e.getMessage());
                return;
            } catch (RemoteException e) {
                System.out.println(e.getMessage());
                return;
            } catch (InvalidParameterFault e) {
                System.out.println(e.getFaultMessage().getMessage());
                return;
            } catch (InternalBaseFault e) {
                System.out.println(e.getFaultMessage().getInternalBaseFault().getMessage());
                return;
            } catch (AccessControlFault e) {
                System.out.println(e.getFaultMessage().getMessage());
                return;
            }
            
            if (response == null) {
                System.out.println("none response received!");
                return;
            }
            
            if (response.isActivityIDSpecified()) {
                String[] responseItems = response.getActivityID();

                for (int i = 0; i < responseItems.length; i++) {
                    System.out.println("activityId = " + responseItems[i]);
                }

                System.out.println("\nitems = " + responseItems.length);  
                System.out.println("is truncated = " + response.getTruncated());  
            } else {
                System.out.println("none activityId found");
            }          
        } else if (isGetActivityStatus()) {
            if (getIdList().size() == 0) {
                printUsage();
                return;
            }

            GetActivityStatus req = new GetActivityStatus();
            req.setActivityID(getIdArray());

            GetActivityStatusResponse response = null;
            try {
                response = getActivityInfoServiceStub().getActivityStatus(req);
            } catch (AxisFault e) {
                System.out.println(e.getMessage());
            } catch (RemoteException e) {
                System.out.println(e.getMessage());
                return;
            } catch (VectorLimitExceededFault e) {
                System.out.println(e.getFaultMessage().getMessage());
                return;
            } catch (InternalBaseFault e) {
                System.out.println(e.getFaultMessage().getInternalBaseFault().getMessage());
                return;
            } catch (AccessControlFault e) {
                System.out.println(e.getFaultMessage().getMessage());
                return;
            }

            ActivityStatus_type0 status = null;
            ActivityStatusItemChoice_type1 choice = null;
            ActivityStatusAttribute[] statusAttributes = null;
            ActivityStatusItem_type0[] activityStatusItems = response.getActivityStatusItem();

            for (int i = 0; i < activityStatusItems.length; i++) {
                StringBuffer buffer = new StringBuffer("activityId = ");
                buffer.append(activityStatusItems[i].getActivityID());
                choice = activityStatusItems[i].getActivityStatusItemChoice_type1();

                if (choice.isActivityStatusSpecified()) {
                    status = choice.getActivityStatus();

                    buffer.append("\nstatus = ").append(status.getStatus().getValue());

                    if (status.isAttributeSpecified()) {
                        buffer.append("\nattributes = {");

                        statusAttributes = status.getAttribute();

                        for (int j = 0; j < statusAttributes.length; j++) {
                            buffer.append(statusAttributes[j].getValue()).append(", ");
                        }

                        buffer.replace(buffer.length() - 2, buffer.length() - 1, "}");
                    }

                    if (status.getDescription() != null) {
                        buffer.append("\ndescription = " + status.getDescription());
                    }

                    buffer.append("\ntimestamp = " + status.getTimestamp().getTime());
                } else {
                    InternalBaseFault_Type fault = choice.getInternalBaseFault();

                    if (fault != null) {
                        buffer.append("\nfault = ").append(fault.getClass().getName());
                        buffer.append("\nmessage = ").append(fault.getMessage());
                        buffer.append("\ntimestamp = " + fault.getTimestamp().getTime());

                        if (fault.getDescription() != null) {
                            buffer.append("\ndescription = " + fault.getDescription());
                        }
                    }
                }

                System.out.println(buffer.toString());
            }
        } else if (isGetActivityInfo()) {
            if (getIdList().size() == 0) {
                printUsage();
                return;
            }

            GetActivityInfo req = new GetActivityInfo();
            req.setActivityID(getIdArray());

            GetActivityInfoResponse response = null;
            try {
                response = getActivityInfoServiceStub().getActivityInfo(req);
            } catch (AxisFault e) {
                System.out.println("AxisFault: " + e.getMessage());
                e.printStackTrace();
            } catch (RemoteException e) {
                System.out.println("RemoteException: " + e.getMessage());
                return;
            } catch (VectorLimitExceededFault e) {
                System.out.println("VectorLimitExceededFault " + e.getFaultMessage().getMessage());
                return;
            } catch (InternalBaseFault e) {
                System.out.println("InternalBaseFault:" + e.getFaultMessage().getInternalBaseFault().getMessage());
                return;
            } catch (AccessControlFault e) {
                System.out.println("AccessControlFault: " + e.getFaultMessage().getMessage());
                return;
            } catch (UnknownAttributeFault e) {
                System.out.println("UnknownAttributeFault: " + e.getFaultMessage().getUnknownAttributeFault().getMessage());
                return;
            } catch (UnknownActivityIDFault e) {
                System.out.println("UnknownActivityIDFault: " + e.getFaultMessage().getUnknownActivityIDFault().getMessage());
                return;
            }

            if (response == null) {
            	System.out.println("none response received!");
            	return;
            }

            ActivityInfoDocument_t activityInfoDocument = null;
            Extensions_t extensions = null;
            ActivityInfoItemChoice_type1 choice = null;
            ActivityInfoItem_type0[] activityInfoItems = response.getActivityInfoItem();

            for (int i = 0; i < activityInfoItems.length; i++) {
                StringBuffer buffer = new StringBuffer("activityId = ");
                buffer.append(activityInfoItems[i].getActivityID());

                choice = activityInfoItems[i].getActivityInfoItemChoice_type1();

                if (choice.isActivityInfoDocumentSpecified()) {
                    activityInfoDocument = choice.getActivityInfoDocument();

                    buffer.append("\nid = ").append(checkValue(activityInfoDocument.getID()));
                    buffer.append("\nidFromEndpoint = ").append(checkValue(activityInfoDocument.getIDFromEndpoint()));
                    buffer.append("\nlocalIDFromManager = ").append(checkValue(activityInfoDocument.getLocalIDFromManager()));
                    buffer.append("\nbaseType = ").append(checkValue(activityInfoDocument.getType()));
                    buffer.append("\ntype = ").append(checkValue(activityInfoDocument.getType().toString()));
                    buffer.append("\nname = ").append(checkValue(activityInfoDocument.getName()));
                    buffer.append("\ndescription = ").append(checkValue(activityInfoDocument.getJobDescription()));
                    buffer.append("\ncomputingManagerExitCode = ").append(checkValue(activityInfoDocument.getComputingManagerExitCode()));
                    buffer.append("\nlocalOwner = ").append(checkValue(activityInfoDocument.getLocalOwner()));
                    buffer.append("\nowner = ").append(checkValue(activityInfoDocument.getOwner()));
                    buffer.append("\nuserDomain = ").append(checkValue(activityInfoDocument.getUserDomain()));
                    buffer.append("\nlogDir = ").append(checkValue(activityInfoDocument.getLogDir()));
                    buffer.append("\nqueue = ").append(checkValue(activityInfoDocument.getQueue()));
                    buffer.append("\nstdIn = ").append(checkValue(activityInfoDocument.getStdIn()));
                    buffer.append("\nstdOut = ").append(checkValue(activityInfoDocument.getStdOut()));
                    buffer.append("\nstdErr = ").append(checkValue(activityInfoDocument.getStdErr()));
                    buffer.append("\nsessionDirectory = ").append(checkValue(activityInfoDocument.getSessionDirectory()));
                    buffer.append("\nstageInDirectory = ").append(checkValue(activityInfoDocument.getStageInDirectory()));
                    buffer.append("\nstageOutDirectory = ").append(checkValue(activityInfoDocument.getStageOutDirectory()));
                    buffer.append("\nsubmissionClientName = ").append(checkValue(activityInfoDocument.getSubmissionClientName()));
                    buffer.append("\nsubmissionHost = ").append(checkValue(activityInfoDocument.getSubmissionHost()));
                    buffer.append("\nexitCode = ").append(checkValue(activityInfoDocument.getExitCode()));
                    buffer.append("\ncreationTime = ").append(checkValue(activityInfoDocument.getCreationTime()));
                    buffer.append("\nstartTime = ").append(checkValue(activityInfoDocument.getStartTime()));
                    buffer.append("\nsubmissionTime = ").append(checkValue(activityInfoDocument.getSubmissionTime()));
                    buffer.append("\ncomputingManagerSubmissionTime = ").append(checkValue(activityInfoDocument.getComputingManagerSubmissionTime()));
                    buffer.append("\ncomputingManagerEndTime = ").append(checkValue(activityInfoDocument.getComputingManagerEndTime()));
                    buffer.append("\nendTime = ").append(checkValue(activityInfoDocument.getEndTime()));
                    buffer.append("\nworkingAreaEraseTime = ").append(checkValue(activityInfoDocument.getWorkingAreaEraseTime()));
                    buffer.append("\nproxyExpirationTime = ").append(checkValue(activityInfoDocument.getProxyExpirationTime()));
                    buffer.append("\nstate = ").append(checkValue(activityInfoDocument.getState()));
                    buffer.append("\nrestartState = ").append(checkValue(activityInfoDocument.getRestartState()));
                    buffer.append("\nerror = ").append(checkValue(activityInfoDocument.getError()));
                    buffer.append("\notherInfo = ").append(checkValue(activityInfoDocument.getOtherInfo()));
                    buffer.append("\notherMessages = ").append(checkValue(activityInfoDocument.getOtherMessages()));
                    buffer.append("\nexecutionNode = ").append(checkValue(activityInfoDocument.getExecutionNode()));
                    buffer.append("\nrequestedApplicationEnvironment = ").append(checkValue(activityInfoDocument.getRequestedApplicationEnvironment()));
                    buffer.append("\nrequestedSlots = ").append(activityInfoDocument.getRequestedSlots());
                    buffer.append("\nrequestedTotalCPUTime = ").append(checkValue(activityInfoDocument.getRequestedTotalCPUTime()));
                    buffer.append("\nrequestedTotalWallTime = ").append(checkValue(activityInfoDocument.getRequestedTotalWallTime()));
                    buffer.append("\nusedMainMemory = ").append(checkValue(activityInfoDocument.getUsedMainMemory()));
                    buffer.append("\nusedTotalCPUTime = ").append(checkValue(activityInfoDocument.getUsedTotalCPUTime()));
                    buffer.append("\nusedTotalWallTime = ").append(checkValue(activityInfoDocument.getUsedTotalWallTime()));
                    buffer.append("\nvalidity = ").append(checkValue(activityInfoDocument.getValidity()));
                    buffer.append("\nwaitingPosition = ").append(activityInfoDocument.getWaitingPosition());

                    extensions = activityInfoDocument.getExtensions();

                    if (extensions != null) {
                        buffer.append("\nextensions = {");

                        for (Extension_t extension : extensions.getExtension()) {
                            buffer.append("\n\t[localID = ").append(checkValue(extension.getLocalID().getLocalID_t()));
                            buffer.append("; key = ").append(checkValue(extension.getKey()));
                            buffer.append("; value = ").append(checkValue(extension.getValue())).append("]");
                            buffer.append("; any = ").append(extension.getExtraElement());
                        }

                        buffer.append("\n}");
                    }
                } else {
                    InternalBaseFault_Type fault = choice.getInternalBaseFault();

                    if (fault != null) {
                        buffer.append("\nfault = ").append(fault.getClass().getName());
                        buffer.append("\nmessage = ").append(fault.getMessage());
                        buffer.append("\ntimestamp = " + fault.getTimestamp().getTime());

                        if (fault.getDescription() != null) {
                            buffer.append("\ndescription = " + fault.getDescription());
                        }
                    }
                }

                System.out.println(buffer.toString());
            }
        } else {
            printUsage();
        }
    }
}
