/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package org.glite.ce.cream.jobmanagement.util;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import org.glite.ce.creamapi.jobmanagement.Job;
import org.glite.ce.creamapi.jobmanagement.JobStatus;
import org.glite.ce.creamapi.jobmanagement.Lease;
import org.glite.ce.creamapi.jobmanagement.command.JobCommand;

/**
 * Class collecting utilities to deal with the <code>Job<code> serialization issues.
 *
 */
public class JobSerialization {
    
    /**
     * Returns a dummy <code>Job</code> instance.
     * @return a dummy <code>Job</code> instance.
     */
    public static Job getDummyJob() {
        Job job = new Job();
        job.setCreamURL("CREAM766697964");
        job.setId("Id_1");
        job.setLRMSJobId("lrmsJobId_1");
        job.setLRMSAbsLayerJobId("lrmsAbsLayerJobId_1");
        job.setGridJobId("gridJobId_1");
        job.setICEId("iceId_1");
        job.setFatherJobId("fatherJobId_1");
        job.setCeId("ceId_1");
        job.setUserId("userId_1");
        job.setJDL("jdl_1");
        job.setWorkerNode("wn_1");
        job.setCeRequirements("cerequirements_1");
        job.setCeRequirements("cerequirements_1");
        job.setVirtualOrganization("vo_1");
        job.setBatchSystem("batchSystem_1");
        job.setQueue("queue_1");
        job.setStandardInput("standardInput_1");
        job.setStandardOutput("standardOutput_1");
        job.setStandardError("standardError_1");
        job.setExecutable("executable_1");
        job.setDelegationProxyCertPath("delegationProxyCertPath_1");
        job.setAuthNProxyCertPath("authNProxyCertPath_1");
        job.setHlrLocation("hlr_1");
        job.setLoggerDestURI("loggerDestURI_1");
        job.setTokenURL("tokenURL_1");
        job.setPerusalFilesDestURI("perusalFilesDestURI_1");
        job.setPerusalListFileURI("perusalListFileURI_1");
        job.setPrologue("prologue_1");
        job.setPrologueArguments("prologueArguments_1");
        job.setEpilogue("epilogue_1");
        job.setEpilogueArguments("epilogueArguments_1");
        job.setSequenceCode("sequenceCode_1");
        job.setType("type_1");
        job.setCREAMInputSandboxURI("CREAMInputSandboxURI_1");
        job.setCREAMOutputSandboxURI("CREAMIOutputSandboxURI_1");
        job.setSandboxBasePath("sandboxBasePath_1");
        job.setInputSandboxBaseURI("inputSandboxBaseURI_1");
        job.setOutputSandboxBaseDestURI("outputSandboxBaseDestURI_1");
        job.setLocalUser("localUser");
        job.setDelegationProxyId("delegationProxy_1");
        job.setDelegationProxyInfo("delegationProxyInfo_1");
        job.setWorkingDirectory("workingDirectory_1");
        String[] strings = new String[3];
        for (int i = 0; i < strings.length; i++) {
            strings[i] = "argument_" + i;
        }
        job.setArguments(strings);
        
        strings = new String[2];
        for (int i = 0; i < strings.length; i++) {
            strings[i] = "child_" + i;
        }
        job.setChildJobId(strings);
        
        strings = new String[2];
        for (int i = 0; i < strings.length; i++) {
            strings[i] = "outputSandboxDestURI_" + i;
        }
        job.setOutputSandboxDestURI(strings);
        
        strings = new String[2];
        for (int i = 0; i < strings.length; i++) {
            strings[i] = "inputFile_" + i;
        }
        job.setInputFiles(strings);

        strings = new String[2];
        for (int i = 0; i < strings.length; i++) {
            strings[i] = "outputFile_" + i;
        }
        job.setOutputFiles(strings);
        
        job.setNodeNumber(10);
        job.setPerusalTimeInterval(1000);
        
        Lease lease = new Lease();
        lease.setLeaseId("leaseId_1");
        lease.setUserId("userId_1");
        lease.setLeaseTime(Calendar.getInstance());
        job.setLease(lease);
        
        Hashtable<String, String> env = new Hashtable<String, String>();
        env.put("key_1", "value_1");
        env.put("key_2", "value_2");
        env.put("key_3", "value_3");
        job.setEnvironment(env);
        
        List<JobStatus> list = new ArrayList<JobStatus>();
        JobStatus s0 = new JobStatus(0);
        list.add(s0);
        JobStatus s1 = new JobStatus(1);
        list.add(s1);
        job.setStatusHistory(list);
        
        List<JobCommand> commands = new ArrayList<JobCommand>();
        JobCommand c0 = new JobCommand(0);
        commands.add(c0);
        JobCommand c1 = new JobCommand(1);
        commands.add(c1);
        job.setCommandHistory(commands);

        return job;
    }
    
    /**
     * Returns the <code>String</code> representing a printing of the information related to a <code>Job</code>.
     * @param job the <code>Job</code> to be printed.
     * @return
     */
    public static String printJob(Job job) {
        StringBuffer sb = new StringBuffer();
        if(job != null) {
            sb.append("version");
            sb.append("=");
            sb.append(Job.version + "\n");
            
            sb.append("creamURL");
            sb.append("=");
            sb.append(job.getCreamURL() + "\n");
            
            sb.append("id");
            sb.append("=");
            sb.append(job.getId() + "\n");
            
            sb.append("lrmsJobId");
            sb.append("=");
            sb.append(job.getLRMSJobId() + "\n");
 
            sb.append("lrmsAbsLayerJobId");
            sb.append("=");
            sb.append(job.getLRMSAbsLayerJobId() + "\n");

            sb.append("gridJobId");
            sb.append("=");
            sb.append(job.getGridJobId() + "\n");
            
            sb.append("iceId");
            sb.append("=");
            sb.append(job.getICEId() + "\n");
            
            sb.append("userId");
            sb.append("=");
            sb.append(job.getUserId() + "\n");
            
            sb.append("jdl");
            sb.append("=");
            sb.append(job.getJDL() + "\n");
            
            sb.append("workerNode");
            sb.append("=");
            sb.append(job.getWorkerNode() + "\n");
            
            sb.append("cerequirements");
            sb.append("=");
            sb.append(job.getCeRequirements() + "\n");
            
            sb.append("virtualOrganization");
            sb.append("=");
            sb.append(job.getVirtualOrganization() + "\n");
            
            sb.append("batchSystem");
            sb.append("=");
            sb.append(job.getBatchSystem() + "\n");
            
            sb.append("queue");
            sb.append("=");
            sb.append(job.getQueue() + "\n");
            
            sb.append("standardInput");
            sb.append("=");
            sb.append(job.getStandardInput() + "\n");
            
            sb.append("standardOutput");
            sb.append("=");
            sb.append(job.getStandardOutput() + "\n");
            
            sb.append("standardError");
            sb.append("=");
            sb.append(job.getStandardError() + "\n");
            
            sb.append("executable");
            sb.append("=");
            sb.append(job.getExecutable() + "\n");
            
            sb.append("delegationProxyCertPath");
            sb.append("=");
            sb.append(job.getDelegationProxyCertPath() + "\n");
            
            sb.append("authNProxyCertPath");
            sb.append("=");
            sb.append(job.getAuthNProxyCertPath() + "\n");
            
            sb.append("hlrLocation");
            sb.append("=");
            sb.append(job.getHlrLocation() + "\n");
            
            sb.append("loggerDestURI");
            sb.append("=");
            sb.append(job.getLoggerDestURI() + "\n");
            
            sb.append("tokenURL");
            sb.append("=");
            sb.append(job.getTokenURL() + "\n");
            
            sb.append("perusalListFileURI");
            sb.append("=");
            sb.append(job.getPerusalListFileURI() + "\n");
            
            sb.append("perusalFilesDestURI");
            sb.append("=");
            sb.append(job.getPerusalFilesDestURI() + "\n");
            
            sb.append("prologue");
            sb.append("=");
            sb.append(job.getPrologue() + "\n");
            
            sb.append("prologueArguments");
            sb.append("=");
            sb.append(job.getPrologueArguments() + "\n");
            
            sb.append("epilogue");
            sb.append("=");
            sb.append(job.getEpilogue() + "\n");
            
            sb.append("epilogueArguments");
            sb.append("=");
            sb.append(job.getEpilogueArguments() + "\n");
            
            sb.append("sequenceCode");
            sb.append("=");
            sb.append(job.getSequenceCode() + "\n");
            
            sb.append("type");
            sb.append("=");
            sb.append(job.getType() + "\n");
            
            sb.append("creamInputSandboxURI");
            sb.append("=");
            sb.append(job.getCREAMInputSandboxURI() + "\n");
            
            sb.append("creamOutputSandboxURI");
            sb.append("=");
            sb.append(job.getCREAMOutputSandboxURI() + "\n");
            
            sb.append("sandboxBasePath");
            sb.append("=");
            sb.append(job.getSandboxBasePath() + "\n");
            
            sb.append("inputSandboxBaseURI");
            sb.append("=");
            sb.append(job.getInputSandboxBaseURI() + "\n");
            
            sb.append("outputSandboxBaseDestURI");
            sb.append("=");
            sb.append(job.getOutputSandboxBaseDestURI() + "\n");
            
            sb.append("localUser");
            sb.append("=");
            sb.append(job.getLocalUser() + "\n");
            
            sb.append("delegationProxyId");
            sb.append("=");
            sb.append(job.getDelegationProxyId() + "\n");
            
            sb.append("delegationProxyInfo");
            sb.append("=");
            sb.append(job.getDelegationProxyInfo() + "\n");
            
            sb.append("workingDirectory");
            sb.append("=");
            sb.append(job.getWorkingDirectory() + "\n");
            
            String[] array = job.getArguments();
            int size = array!= null ? array.length : -1;
            sb.append("arguments_size");
            sb.append("=");
            sb.append(size + "\n");
            if (array != null) {
                for (int i = 0; i < array.length; i++) {
                    sb.append("val[" + i + "]");
                    sb.append("=");
                    sb.append(array[i] + "\n");
                }
            }
            
            array = job.getChildJobId();
            size = array!= null ? array.length : -1;
            sb.append("childJobIds_size");
            sb.append("=");
            sb.append(size + "\n");
            if (array != null) {
                for (int i = 0; i < array.length; i++) {
                    sb.append("val[" + i + "]");
                    sb.append("=");
                    sb.append(array[i] + "\n");
                }
            }
            
            array = job.getOutputSandboxDestURI();
            size = array!= null ? array.length : -1;
            sb.append("outputSandboxDestURIs_size");
            sb.append("=");
            sb.append(size + "\n");
            if (array != null) {
                for (int i = 0; i < array.length; i++) {
                    sb.append("val[" + i + "]");
                    sb.append("=");
                    sb.append(array[i] + "\n");
                }
            }
            
            array = job.getInputFiles();
            size = array!= null ? array.length : -1;
            sb.append("inputFiles_size");
            sb.append("=");
            sb.append(size + "\n");
            if (array != null) {
                for (int i = 0; i < array.length; i++) {
                    sb.append("val[" + i + "]");
                    sb.append("=");
                    sb.append(array[i] + "\n");
                }
            }
            
            array = job.getOutputFiles();
            size = array!= null ? array.length : -1;
            sb.append("outputFiles_size");
            sb.append("=");
            sb.append(size + "\n");
            if (array != null) {
                for (int i = 0; i < array.length; i++) {
                    sb.append("val[" + i + "]");
                    sb.append("=");
                    sb.append(array[i] + "\n");
                }
            }
            
            sb.append("nodes");
            sb.append("=");
            sb.append(job.getNodeNumber() + "\n");
            
            sb.append("perusalTimeInterval");
            sb.append("=");
            sb.append(job.getPerusalTimeInterval() + "\n");
            
            Lease lease = job.getLease();
            sb.append("lease");
            sb.append("=");
            if (lease != null) {
                sb.append("leaseId");
                sb.append("=");
                sb.append(lease.getLeaseId() + "\n");
                
                sb.append("leaseTime");
                sb.append("=");
                long time = lease.getLeaseTime() != null ? lease.getLeaseTime().getTimeInMillis() : -1;
                sb.append(time+ "\n");
                
                sb.append("userId");
                sb.append("=");
                sb.append(lease.getUserId() + "\n");
            } else {
                sb.append(lease);
            }
            
            sb.append("environment");
            sb.append("=");
            Hashtable<String, String> env = job.getEnvironment();
            size = env!= null ? env.size() : -1;
            sb.append("environment_size");
            sb.append("=");
            sb.append(size + "\n");
            if (env != null) {
            Enumeration<String> allKeys = env.keys();
                while (allKeys.hasMoreElements()) {
                    String key = allKeys.nextElement();
                    sb.append("[" + key + ", " + env.get(key) + "]\n");
                }
            }
            
            List<JobStatus> list = job.getStatusHistory();
            size = list!= null ? list.size() : -1;
            sb.append("statusHistory_size");
            sb.append("=");
            sb.append(size + "\n");
            if (list != null) {
                for (int i = 0; i < list.size(); i++) {
                    sb.append("type=" + list.get(i).getType() + "\n");
                    sb.append("name=" + list.get(i).getName() + "\n");
                    sb.append("description=" + list.get(i).getDescription() + "\n");
                    sb.append("exitCode=" + list.get(i).getExitCode() + "\n");
                    sb.append("failureReason=" + list.get(i).getFailureReason() + "\n");
                    sb.append("jobId=" + list.get(i).getJobId() + "\n");
                    long time = list.get(i).getTimestamp() != null ? list.get(i).getTimestamp().getTimeInMillis() : -1;
                    sb.append("timestamp=" + time + "\n");
                }
            }
            
            List<JobCommand> commands = job.getCommandHistory();
            size = commands!= null ? commands.size() : -1;
            sb.append("commandHistory_size");
            sb.append("=");
            sb.append(size + "\n");
            if (commands != null) {
                for (int i = 0; i < commands.size(); i++) {
                    sb.append("executorName=" + commands.get(i).getCommandExecutorName() + "\n");
                    commands.get(i).getCreationTime();
                    long time = commands.get(i).getExecutionCompletedTime() != null ? commands.get(i).getExecutionCompletedTime().getTimeInMillis() : -1;
                    sb.append("executionCompletedTime=" + time + "\n");

                    time = commands.get(i).getStartProcessingTime() != null ? commands.get(i).getStartProcessingTime().getTimeInMillis() : -1;
                    sb.append("startProcessingTime=" + time + "\n");
                    
                    time = commands.get(i).getStartSchedulingTime() != null ? commands.get(i).getStartSchedulingTime().getTimeInMillis() : -1;
                    sb.append("startSchedulingTime=" + time + "\n");
                    
                    time = commands.get(i).getCreationTime() != null ? commands.get(i).getCreationTime().getTimeInMillis() : -1;
                    sb.append("creationTime=" + time + "\n");
                    sb.append("failureReason=" + commands.get(i).getFailureReason() + "\n");
                    sb.append("jobId=" + commands.get(i).getJobId() + "\n");
                    sb.append("name=" + commands.get(i).getName() + "\n");
                    sb.append("description=" + commands.get(i).getDescription() + "\n");
                    sb.append("status=" + commands.get(i).getStatus() + "\n");
                    sb.append("statusName=" + commands.get(i).getStatusName() + "\n");
                    sb.append("type=" + commands.get(i).getType() + "\n");
                    sb.append("userId=" + commands.get(i).getUserId() + "\n");
                }
            }
        } else {
            sb.append("Job is NULL");
        }
        return sb.toString();        
    }
}
