/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 * 
 */

package org.glite.ce.cream.cmdmanagement.db;

import java.util.GregorianCalendar;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.creamapi.cmdmanagement.Command;
import org.glite.ce.creamapi.cmdmanagement.CommandException;


/**
 * Class to test command management with respect to the dB.
 *
 */
public final class CommandTest {
   
	public static void main(String[] args) throws Exception {
	    test1();
	}
	
	/**
	 * Tests simple CRUD operations on commands.
	 * - insert
	 * - insert
	 * - update (set true the scheduled flag)
	 * - load
	 * - delete
	 * @throws DatabaseException
	 * @throws IllegalArgumentException
	 * @throws CommandException
	 */
	private static void test1() throws DatabaseException, IllegalArgumentException, CommandException {
	    CommandDBManager cmdManager = new CommandDBManager();
        Command cmd = CommandTestUtil.getCommand("userX");
        cmdManager.insertCommand(cmd);
        System.out.println("Inserted command with cmdId = " + cmd.getId());
        
        cmdManager.insertCommand(cmd);
        System.out.println("Inserted command with cmdId = " + cmd.getId());
        
        cmd.setStatus(Command.SCHEDULED);
        cmd.setStartSchedulingTime(new GregorianCalendar());
        cmdManager.updateCommand(cmd);
        System.out.println("Updated command with cmdId = " + cmd.getId());
          
        cmd = cmdManager.getCommand(cmd.getId());
        System.out.println("Selected command with cmdId = " + cmd.getId());
        
        cmdManager.deleteCommand(cmd.getId());
        System.out.println("Deleted command with cmdId = " + cmd.getId());
	}
}
