/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 *
 * Authors: Eric Frizziero <eric.frizziero@pd.infn.it>
 * 
 */

package org.glite.ce.cream.jobmanagement.db;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.creamapi.jobmanagement.Job;
import org.glite.ce.creamapi.jobmanagement.JobStatus;
import org.glite.ce.creamapi.jobmanagement.Lease;
import org.glite.ce.creamapi.jobmanagement.command.JobCommand;
import org.glite.ce.creamapi.jobmanagement.db.JobDBInterface;

public class JobDBTest {
	JobDBInterface jobDB = null;
	
	public JobDBTest(){
		jobDB = new JobDBImplementation();
		jobDB.init(null);
	}
	
	public String insertTest(String userId, String delegationId) throws DatabaseException{
		Job job = createJobTest(userId, delegationId);
		jobDB.insert(job);
		return job.getId();
	}
	
	public void deleteTest(String jobId, String userId) throws DatabaseException{
		jobDB.delete(jobId, userId);
	}
	
	public Job createJobTest(String userId, String delegationId){
		  Job job = new Job();
		  job.setUserId(userId);
		  job.setDelegationProxyId(delegationId);
		  //Job.setCreamURL("creamurl");
		  job.setBatchSystem("BatchSystem");
		  job.setQueue("Queue");
		  job.setExecutable("Executable");
		  job.setType("Type");
		  job.setJDL("JDL");
		  job.setLocalUser("LocalUser");
		  job.setWorkingDirectory("WorkingDirectory");
		  Lease lease = new Lease();
		  lease.setUserId(userId);
		  lease.setLeaseId("LeaseIdPIPPO");
		  lease.setLeaseTime(Calendar.getInstance());
		  job.setLease(lease);
		return job;
	  }

	public List<String> retrieveJobIdTest(String userId) throws DatabaseException{
		return jobDB.retrieveJobId(userId);
	}
	
	public List<String> retrieveJobIdTest(String userId, List<String> jobId) throws DatabaseException{
		return jobDB.retrieveJobId(jobId, userId);
	}
	
	public List<String> retrieveJobIdTest(String userId, List<String> jobId, int[] jobStatusType) throws DatabaseException{
		return jobDB.retrieveJobId(jobId, userId, jobStatusType);
	}
	
	
	public Job retrieveJob(String jobId, String userId) throws DatabaseException{
		return jobDB.retrieveJob(jobId, userId);
	}

	public JobStatus retrieveLastJobStatus(String jobId, String userId)throws DatabaseException{
		JobStatus jobStatus = null;
		jobStatus = jobDB.retrieveLastJobStatus(jobId, userId);
		return jobStatus;
	}
	
	public void updateLastStatus(JobStatus status, String userId)throws DatabaseException{
		jobDB.updateLastStatus(status, userId);
	}
	
	
	public void insertStatus(JobStatus status, String userId) throws DatabaseException{
		jobDB.insertStatus(status, userId);
	}
	
	public List<JobStatus> retrieveJobStatusHistory(String jobId, String userId) throws DatabaseException{
		return jobDB.retrieveJobStatusHistory(jobId, userId);
	}
	
	public JobCommand retrieveLastJobCommand(String jobId, String userId)throws DatabaseException{
		JobCommand jobCommand = null;
		jobCommand = jobDB.retrieveLastCommand(jobId, userId);
		return jobCommand;
	}
	
	public void updateLastJobCommand(JobCommand jobCommand)throws DatabaseException{
		jobDB.updateLastJobCommand(jobCommand);
	}
	
	
	public void insertJobCommand(JobCommand jobCommand) throws DatabaseException{
		jobDB.insertJobCommand(jobCommand);
	}
	
	public List<JobCommand> retrieveJobCommandHistory(String jobId, String userId) throws DatabaseException{
		return jobDB.retrieveCommandHistory(jobId, userId);
	}
	
	public void update(Job job)throws DatabaseException{
		jobDB.update(job);
	}
	
	 public static void main(String[] args) throws Exception{
		 String userId = "EEE";
		 String delegationId = "delegationPippo";
		 JobDBTest jobDBTest = new JobDBTest(); 

		 
		 jobDBTest.insertTest(userId, delegationId);
		 
		 List<String> jobIdList = jobDBTest.retrieveJobIdTest(userId);
		 System.out.println("Num job retrieved for " + userId +  " = " + jobIdList.size());
		 for (int i = 0; i<jobIdList.size(); i++){
			 System.out.println("jobId = " + jobIdList.get(i));
		 }
		 jobIdList = jobDBTest.retrieveJobIdTest(userId, jobIdList);
		 System.out.println("Num job retrieved = " + jobIdList.size());
		 for (int i = 0; i<jobIdList.size(); i++){
			 System.out.println("jobId = " + jobIdList.get(i));
		 }
		 
		 Job retrievedJob = null;
		 retrievedJob  = jobDBTest.retrieveJob(jobIdList.get(0), userId);
		 System.out.println("Retrieved DelegationId (must be '" + delegationId + "') = " + retrievedJob.getDelegationProxyId());
		 System.out.println("Retrieved jobId  = " + retrievedJob.getId());
		 System.out.println("Retrieved userId  = " + retrievedJob.getUserId());
		 
		 System.out.println("updating job ...");
		 retrievedJob.setDelegationProxyId(delegationId + " - " + delegationId);
		 jobDBTest.update(retrievedJob);
		 System.out.println("Retrieved DelegationId (must be '" + delegationId + " - " + delegationId + "') = " + retrievedJob.getDelegationProxyId());
		 
		 
		 System.out.println(">>>>>>>>> START TEST .... STATUS");
		 JobStatus jobStatus = jobDBTest.retrieveLastJobStatus(retrievedJob.getId(), userId);
		 if (jobStatus!=null){
		   System.out.println("Retrieved last job status. ExitCode = " + jobStatus.getExitCode());
		 } else {
			 System.out.println("No status are available!");
		 }
		 
		 System.out.println("Trying update last status ... a DatabaseException must be thrown ...");
		 jobStatus = new JobStatus(3);
		 jobStatus.setExitCode("4");
		 jobStatus.setJobId(retrievedJob.getId());
		 try{
		 jobDBTest.updateLastStatus(jobStatus, userId);
		 } catch(DatabaseException e){
			 System.out.println("Excetion occured: " + e.getMessage());
		 }
		 
		 System.out.println("Trying retrieveJobStatusHistory ... for jobId = " + retrievedJob.getId());
		 List<JobStatus> jobStatusList = jobDBTest.retrieveJobStatusHistory(retrievedJob.getId(), userId);
		 if (jobStatusList.size()>0){
		   for (int i=0; i<jobStatusList.size(); i++){
			 System.out.println("Status num " + i + " ExitCode = " + jobStatusList.get(i).getExitCode());
		   }
		 } else {
			 System.out.println("retrieveJobStatusHistory is empty!");
		 }
		 
		 System.out.println("inserting new status ...");
		 jobStatus = new JobStatus(2);
		 jobStatus.setExitCode("9");
		 jobStatus.setJobId(retrievedJob.getId());
		 
		 jobDBTest.insertStatus(jobStatus, userId);
		 
		 jobStatus = jobDBTest.retrieveLastJobStatus(retrievedJob.getId(), userId);
		 if (jobStatus!=null){
			   System.out.println("Retrieved last job status. ExitCode (must be 9) = " + jobStatus.getExitCode());
		 } else {
			 System.out.println("No status are available!");
		 }
		 System.out.println("now, inserting 4 status");
		 jobStatus = new JobStatus(8);
		 jobStatus.setJobId(retrievedJob.getId());
		 jobStatus.setExitCode("8");
		 jobDBTest.insertStatus(jobStatus, userId);
		 jobStatus = new JobStatus(7);
		 jobStatus.setJobId(retrievedJob.getId());
		 jobStatus.setExitCode("7");
		 jobDBTest.insertStatus(jobStatus, userId);
		 jobStatus = new JobStatus(6);
		 jobStatus.setJobId(retrievedJob.getId());
		 jobStatus.setExitCode("6");
		 jobDBTest.insertStatus(jobStatus, userId);
		 jobStatus = new JobStatus(5);
		 jobStatus.setJobId(retrievedJob.getId());
		 jobStatus.setExitCode("5");
		 jobDBTest.insertStatus(jobStatus, userId);
		 jobStatus = jobDBTest.retrieveLastJobStatus(retrievedJob.getId(), userId);
		 if (jobStatus!=null){
			   System.out.println("Retrieved last job status. Type (must be 5) = " + jobStatus.getExitCode());
		 } else {
			 System.out.println("No status are available!");
		 }
		 System.out.println("updating last status ...");
		 jobStatus.setExitCode("4");
		 jobDBTest.updateLastStatus(jobStatus, userId);
		 jobStatus = jobDBTest.retrieveLastJobStatus(retrievedJob.getId(), userId);
		 if (jobStatus!=null){
			   System.out.println("Retrieved last job status. Type (must be 4) = " + jobStatus.getExitCode());
		 } else {
			 System.out.println("No status are available!");
		 }
		 System.out.println("retrieveJobStatusHistory ... for jobId = " + retrievedJob.getId());
		 jobStatusList = jobDBTest.retrieveJobStatusHistory(retrievedJob.getId(), userId);
		 for (int i=0; i<jobStatusList.size(); i++){
			 System.out.println("Status num " + i + " ExitCode = " + jobStatusList.get(i).getExitCode());
		 }
		 
		 /*
		 System.out.println("updateJobStatusHistory ... for jobId = " + retrievedJob.getId());
		 for (int i=0; i<jobStatusList.size(); i++){
			 jobStatusList.get(i).setExitCode("Modified");
		 }
		 retrievedJob.setStatusHistory(jobStatusList);
		 jobDBTest.update(retrievedJob);
		 
		 jobStatusList = jobDBTest.retrieveJobStatusHistory(retrievedJob.getId(), userId);
		 for (int i=0; i<jobStatusList.size(); i++){
			 System.out.println("Status num " + i + " ExitCode (must be 'Modified') = " + jobStatusList.get(i).getExitCode());
		 }
		 
		 System.out.println("updateJobStatusHistory ... for jobId = " + retrievedJob.getId() + " now DatabaseException must be thrown");
		 JobStatus jobStatusToDelete = jobStatusList.get(jobStatusList.size()-1);
		 try{
		   jobStatusList.remove(jobStatusList.size()-1);
		   retrievedJob.setStatusHistory(jobStatusList);
		   jobDBTest.update(retrievedJob);
		   
		 }catch(DatabaseException de){
			 System.out.println("Excetion = " + de.getMessage());
		 }
		 jobStatusList.add(jobStatusToDelete);
		 retrievedJob.setStatusHistory(jobStatusList);
		 */
		 System.out.println(">>>>>>>>> END TEST .... STATUS");
		 
		 
		 
		 System.out.println(">>>>>>>>> START TEST .... COMMAND");
		 JobCommand jobCommand = jobDBTest.retrieveLastJobCommand(retrievedJob.getId(), userId);
		 if (jobCommand!=null){
		   System.out.println("Retrieved last job Command. Type = " + jobCommand.getType());
		 } else {
			 System.out.println("No jobCommand are available!");
		 }
		 
		 System.out.println("Trying update last command ... a DatabaseException must be thrown ...");
		 jobCommand = new JobCommand(3);
		 jobCommand.setJobId(retrievedJob.getId());
		 jobCommand.setUserId(userId);
		 try{
		 jobDBTest.updateLastJobCommand(jobCommand);
		 } catch(DatabaseException e){
			 System.out.println("Excetion occured: " + e.getMessage());
		 }
		 
		 
		 System.out.println("retrieveJobCommandHistory ... for jobId = " + retrievedJob.getId());
		 List<JobCommand> jobCommandList = jobDBTest.retrieveJobCommandHistory(retrievedJob.getId(), userId);
		 if (jobCommandList.size()>0){
		   for (int i=0; i<jobCommandList.size(); i++){
			 System.out.println("Command num " + i + " Type = " + jobCommandList.get(i).getType());
		   }
		 } else {
			 System.out.println("retrieveJobCommandHistory is empty!");
		 }
		 
		 System.out.println("inserting new jobCommand ...");
		 jobCommand = new JobCommand(2);
		 jobCommand.setUserId(userId);
		 jobCommand.setJobId(retrievedJob.getId());
		 
		 jobDBTest.insertJobCommand(jobCommand);
		 
		 jobCommand = jobDBTest.retrieveLastJobCommand(retrievedJob.getId(), userId);
		 if (jobCommand!=null){
			   System.out.println("Retrieved last job Command. type (must be 2) = " + jobCommand.getType());
		 } else {
			 System.out.println("No jobCommand are available!");
		 }
		 System.out.println("now, inserting 4 jobCommand");
		 jobCommand = new JobCommand(8);
		 jobCommand.setJobId(retrievedJob.getId());
		 jobCommand.setUserId(userId);
		 jobDBTest.insertJobCommand(jobCommand);
		 jobCommand = new JobCommand(7);
		 jobCommand.setJobId(retrievedJob.getId());
		 jobCommand.setUserId(userId);
		 jobDBTest.insertJobCommand(jobCommand);
		 jobCommand = new JobCommand(6);
		 jobCommand.setJobId(retrievedJob.getId());
		 jobCommand.setUserId(userId);
		 jobDBTest.insertJobCommand(jobCommand);
		 jobCommand = new JobCommand(5);
		 jobCommand.setJobId(retrievedJob.getId());
		 jobCommand.setUserId(userId);
		 jobDBTest.insertJobCommand(jobCommand);
		 jobCommand = jobDBTest.retrieveLastJobCommand(retrievedJob.getId(), userId);
		 if (jobCommand!=null){
			   System.out.println("Retrieved last jobCommand. Type (must be 5) = " + jobCommand.getType());
		 } else {
			 System.out.println("No jobCommand are available!");
		 }
		 System.out.println("updating last jobCommand ...");
		 jobCommand.setType(4);
		 jobDBTest.updateLastJobCommand(jobCommand);
		 jobCommand = jobDBTest.retrieveLastJobCommand(retrievedJob.getId(), userId);
		 if (jobStatus!=null){
			   System.out.println("Retrieved last jobCommand. Type (must be 4) = " + jobCommand.getType());
		 } else {
			 System.out.println("No jobCommand are available!");
		 }
		 System.out.println("retrieveJobCommandHistory ... for jobId = " + retrievedJob.getId());
		 jobCommandList = jobDBTest.retrieveJobCommandHistory(retrievedJob.getId(), userId);
		 for (int i=0; i<jobCommandList.size(); i++){
			 System.out.println("Command num " + i + " Type = " + jobCommandList.get(i).getType());
		 }
		 
		 /*
		 System.out.println("updateJobCommandHistory ... for jobId = " + retrievedJob.getId());
		 for (int i=0; i<jobCommandList.size(); i++){
			 jobCommandList.get(i).setType(1);
		 }
		 retrievedJob.setCommandHistory(jobCommandList);
		 jobDBTest.update(retrievedJob);
		 
		 jobCommandList = jobDBTest.retrieveJobCommandHistory(retrievedJob.getId(), userId);
		 for (int i=0; i<jobCommandList.size(); i++){
			 System.out.println("Command num " + i + " Type (must be '1') = " + jobCommandList.get(i).getType());
		 }
		 System.out.println("updateJobCommandHistory ... for jobId = " + retrievedJob.getId() + " now DatabaseException must be thrown");
		 try{
		   jobCommandList.remove(0);
		   retrievedJob.setCommandHistory(jobCommandList);
		   jobDBTest.update(retrievedJob);
		   
		 }catch(DatabaseException de){
			 System.out.println("Excetion = " + de.getMessage());
		 }
		 
		 */
		 System.out.println(">>>>>>>>> END TEST .... COMMAND");
		 
		 System.out.println(">>>>>>>>> START TEST .... RETRIEVE");
		 
         List<String> jobIdInserted = new ArrayList<String>();
         
		 String useridForRetrieving = "XYZ";
		 JobStatus jobStatusForRetrieving = null;
		 // job 1 last status = 1
		 String jobId1 = jobDBTest.insertTest(useridForRetrieving, null);
		 jobIdInserted.add(jobId1);
		 jobStatusForRetrieving = new JobStatus(2);
		 jobStatusForRetrieving.setJobId(jobId1);
		 jobDBTest.insertStatus(jobStatusForRetrieving, useridForRetrieving);
		 jobStatusForRetrieving = new JobStatus(1);
		 jobStatusForRetrieving.setJobId(jobId1);
		 jobDBTest.insertStatus(jobStatusForRetrieving, useridForRetrieving);
		 // job 2 last status = 4
		 String jobId2 = jobDBTest.insertTest(useridForRetrieving, null);
		 jobIdInserted.add(jobId2);
   	     jobStatusForRetrieving = new JobStatus(2);
		 jobStatusForRetrieving.setJobId(jobId2);
		 jobDBTest.insertStatus(jobStatusForRetrieving, useridForRetrieving);
		 jobStatusForRetrieving = new JobStatus(4);
		 jobStatusForRetrieving.setJobId(jobId2);
		 jobDBTest.insertStatus(jobStatusForRetrieving, useridForRetrieving);
		 
		 //job 3 last status = 1
		 String jobId3 = jobDBTest.insertTest(useridForRetrieving, null);
		 jobIdInserted.add(jobId3);
		 jobStatusForRetrieving = new JobStatus(2);
		 jobStatusForRetrieving.setJobId(jobId3);
		 jobDBTest.insertStatus(jobStatusForRetrieving, useridForRetrieving);
		 jobStatusForRetrieving = new JobStatus(1);
		 jobStatusForRetrieving.setJobId(jobId3);
		 jobDBTest.insertStatus(jobStatusForRetrieving, useridForRetrieving);

		 int[] jobStatusType = {1};
		 List<String> jobIdRetrieved = jobDBTest.retrieveJobIdTest(useridForRetrieving, jobIdInserted, jobStatusType);
		 System.out.println("JobId retrieved must be: " + jobId1 + ", " + jobId3);
		 for (int i = 0; i<jobIdRetrieved.size(); i++){
			 System.out.println("jobId retrieved = " + jobIdRetrieved.get(i));
		 }
		 
		 int[] jobStatusType2 = {4};
		 jobIdRetrieved = jobDBTest.retrieveJobIdTest(useridForRetrieving, jobIdInserted, jobStatusType2);
		 System.out.println("JobId retrieved must be: " + jobId2);
		 for (int i = 0; i<jobIdRetrieved.size(); i++){
			 System.out.println("jobId retrieved = " + jobIdRetrieved.get(i));
		 }
		 
		 int[] jobStatusType3 = {1, 4};
		 jobIdRetrieved = jobDBTest.retrieveJobIdTest(useridForRetrieving, jobIdInserted, jobStatusType3);
		 System.out.println("JobId retrieved must be: " + jobId1 + ", " + jobId2 + ", " + jobId3);
		 for (int i = 0; i<jobIdRetrieved.size(); i++){
			 System.out.println("jobId retrieved = " + jobIdRetrieved.get(i));
		 }
		 		 
		 System.out.println(">>>>>>>>> END TEST .... RETRIEVE");
		 
	 }
}
