/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 *
 * Authors: Eric Frizziero <eric.frizziero@pd.infn.it>
 * 
 */

package org.glite.ce.cream.jobmanagement.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.List;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.common.db.DatasourceManager;
import org.glite.ce.cream.jobmanagement.db.table.LeaseTable;
import org.glite.ce.creamapi.jobmanagement.Lease;
import org.glite.ce.creamapi.jobmanagement.db.table.LeaseTableInterface;

public class LeaseTableTest {
	String datasourceName = null;
	LeaseTableInterface   leaseTable   = null;
	
	public LeaseTableTest()throws Exception{
	  this.leaseTable   = new LeaseTable();
	}
	
	public int executeInsertTest(Lease jobLease)throws SQLException, DatabaseException{
		int rowInserted = -1;
		Connection connection = null;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
		      rowInserted = leaseTable.executeInsert(jobLease, connection);
		      connection.commit();
		     // System.out.println("rowInserted = " + rowInserted);
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
			  throw sqle;
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  } 
		return rowInserted;
	}
	
	 public List<Lease> executeSelectTest(String leaseId, String userId, Calendar leaseTime) throws SQLException, DatabaseException {
		 Connection connection = null;
		 List<Lease> leaseList = null;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
			  leaseList = leaseTable.executeSelect(leaseId, userId, leaseTime, connection);
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
			  throw sqle;
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		  return leaseList; 
	 }
	 

	 public int executeDeleteTest(String leaseId, String userId) throws SQLException, DatabaseException {
		 Connection connection = null;
		 int rowDeleted = -1;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
		      rowDeleted = leaseTable.executeDelete(leaseId, userId, connection);
		      connection.commit();
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
			  throw sqle;
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		  return rowDeleted; 
	 }
	 
	 public int executeUpdateTest(Lease jobLease) throws SQLException, DatabaseException {
		 Connection connection = null;
		 int rowUpdated = -1;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
			  rowUpdated = leaseTable.executeUpdate(jobLease, connection);
		      connection.commit();
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
			  throw sqle;
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		  return rowUpdated; 
	 }
	 
	 
	  public static void main(String[] args) throws Exception{
		  LeaseTableTest leaseTableTest = new LeaseTableTest();
		  
		  List<Lease> leaseList = null;
		  
		  String   leaseId   = "leasePippo";
		  String   userId    = "Pippo" + Math.random();
		  Calendar leaseTime = Calendar.getInstance();
		  leaseTime.set(2007, 12, 11);
		  Lease lease =  null;
		  //TEST INSERT
		  lease = new Lease();
		  lease.setLeaseId(leaseId);
		  lease.setLeaseTime(leaseTime);
		  lease.setUserId(userId);
		  int rowInserted = leaseTableTest.executeInsertTest(lease);
		  System.out.println("rowInserted = " + rowInserted);

          // TEST select
		  leaseList = leaseTableTest.executeSelectTest(leaseId, userId, leaseTime);
		  
		  if (leaseList == null){
			  System.out.println("leaseList is NULL!");
		  }
		  if (leaseList.size() == 0){
			  System.out.println("No lease found!");
		  }else{
			  System.out.println("Found " + leaseList.size() + " leases");
			  for (int i=0;i<leaseList.size();i++){
				  lease = leaseList.get(i);
				  System.out.println("Lease num " + i);
				  System.out.println("LeaseId   = " + lease.getLeaseId());
				  System.out.println("UserId    = " + lease.getUserId());
				  System.out.println("LeaseTime = " + lease.getLeaseTime());
			  }
		  }
          
		  // TEST update
		  Calendar newLeaseTime = Calendar.getInstance();
		  newLeaseTime.set(1970, 11, 13);
		  lease.setLeaseTime(newLeaseTime);
		  int rowUpdated = leaseTableTest.executeUpdateTest(lease);
		  System.out.println("rowUpdated = " + rowUpdated);
		  
		  //TEST delete
		  int rowDeleted = leaseTableTest.executeDeleteTest(leaseId, userId);
		  System.out.println("rowDeleted = " + rowDeleted);
	  }
}

